@@ -3,7 +3,7 @@ package Params::ValidationCompiler::Compiler;
 use strict;
 use warnings;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 use Carp qw( croak );
 use Eval::Closure qw( eval_closure );
@@ -416,9 +416,11 @@ sub _compile_positional_args_check {
         unless $self->slurpy;
 
     my $access_var = '$_';
+    my $return_var = '@_';
     if ( $self->_any_type_has_coercion ) {
         push @{ $self->_source }, 'my @copy = @_;';
         $access_var = '$copy';
+        $return_var = '@copy';
     }
 
     for my $i ( 0 .. $#specs ) {
@@ -445,7 +447,7 @@ sub _compile_positional_args_check {
         );
     }
 
-    push @{ $self->_source }, 'return @_;';
+    push @{ $self->_source }, sprintf( 'return %s;', $return_var );
 
     return;
 }
@@ -755,8 +757,8 @@ sub _add_specio_check {
 
         push @source,
             sprintf(
-            '$types{%s}->validate_or_die(%s);', $name,
-            $access
+            '$types{%s}->validate_or_die(%s);',
+            $qname, $access,
             );
 
         $self->_env->{'%types'}{$name} = $type;
@@ -860,7 +862,7 @@ Params::ValidationCompiler::Compiler - Object that implements the check subrouti
 
 =head1 VERSION
 
-version 0.18
+version 0.19
 
 =for Pod::Coverage .*
 
